pub mod impls;

use std::ffi::{OsStr, OsString};
use std::fmt;

pub trait FromOsStr: Sized {
    type Err;

    fn from_os_str(s: &OsStr) -> Result<Self, Self::Err>;
}

pub trait ToOsString {
    fn to_os_string(&self) -> OsString;
}

pub struct OsStringFormatter {
    inner: OsString,
}

impl OsStringFormatter {
    pub fn new() -> Self {
        OsStringFormatter {
            inner: OsString::new(),
        }
    }

    pub fn write_os_str(&mut self, s: &OsStr) -> fmt::Result {
        self.inner.push(s);
        Ok(())
    }

    pub fn write_str(&mut self, s: &str) -> fmt::Result {
        self.inner.push(s);
        Ok(())
    }

    pub fn into_os_string(self) -> OsString {
        self.inner
    }
}

impl Default for OsStringFormatter {
    fn default() -> Self {
        Self {
            inner: OsString::new(),
        }
    }
}

pub trait OsDisplay {
    fn fmt_os(&self, f: &mut OsStringFormatter) -> fmt::Result;
}

impl<T: OsDisplay> ToOsString for T {
    fn to_os_string(&self) -> OsString {
        let mut formatter = OsStringFormatter::new();
        match self.fmt_os(&mut formatter) {
            Ok(_) => formatter.into_os_string(),
            Err(_) => OsString::new(),
        }
    }
}
